

float2 bezier_pt_deg1(__read_only __global float2* ctrl_point, const float t)
{
	return ctrl_point[0] * (1.0f-t) + ctrl_point[1] * t;
}
float2 bezier_pt_deg2(__read_only __global float2* V, const float t)
{
		const float t2 = t*t;
		const float t12 = 2.0f*t;
	return V[0] * (t2 - t12 + 1.0f) + V[1] * (t12 - 2.0f * t2) + V[2] * t2;
}
float2 bezier_pt_deg3(__read_only __global float2* V, const float t)
{
		const float t2 = t*t;
		const float t3 = t2*t;
		const float _3t = t * 3.0f;
		const float _3t2 = t2 * 3.0f;
		const float _3t3 = t3 * 3.0f;
	return V[0] * (-t3 + _3t2 - _3t + 1.0f) + V[1] * (_3t3 - 2.0f * _3t2 + _3t) + V[2] * (-_3t3 + _3t2) + V[3] * t3;
}




float get_error_deg1(const float2 point, __read_only __global float2* ctrl_point)
{
	float error;
	float t_left = 0.0f, t_right = 1.0f;
	float t_middle = 0.5f;
	float t_left_middle = 0.25f;
	float t_right_middle = 0.75f;

	float error_middle = fast_length(bezier_pt_deg1(ctrl_point,t_middle) - point);
	float error_left, error_right;
	int max_iter = 100;
	int iter = 0;
	do
	{
		error_left = fast_length(bezier_pt_deg1(ctrl_point,t_left_middle)- point);
		error_right = fast_length(bezier_pt_deg1(ctrl_point,t_right_middle)- point);//fn(t_right_middle);
		if (error_left + 0.001f < error_right && error_left + 0.001f < error_middle)
		{
			t_right = t_middle;
			t_middle = t_left_middle;
			error_middle = error_left;
		}
		else if (error_middle + 0.001f < error_right && error_middle + 0.001f < error_left)
		{
			t_right = t_right_middle;
			t_left = t_left_middle;
		}
		else if (error_right + 0.001f < error_middle && error_right + 0.001f < error_left)
		{
			t_left = t_middle;
			t_middle = t_right_middle;
			error_middle = error_right;
		}
		else
		{
			break;//2 are equals
		}
		t_left_middle = (t_left + t_middle)*0.5f;
		t_right_middle = (t_right + t_middle)*0.5f;
		++iter;
	} while (iter < max_iter);// t_left_middle>0.1 && t_right_middle < 0.9);
	return fast_length(bezier_pt_deg1(ctrl_point,t_middle)- point);
}
float get_error_deg2(float2 point, __read_only __global float2* ctrl_point)
{
	float error;
	float t_left = 0.0f, t_right = 1.0f;
	float t_middle = 0.5f;
	float t_left_middle = 0.25f;
	float t_right_middle = 0.75f;

	float error_middle = fast_length(bezier_pt_deg2(ctrl_point,t_middle)- point);
	float error_left, error_right;
	int max_iter = 100;
	int iter = 0;
	do
	{
		error_left = fast_length(bezier_pt_deg2(ctrl_point,t_left_middle)- point);
		error_right = fast_length(bezier_pt_deg2(ctrl_point,t_right_middle)- point);//fn(t_right_middle);
		if (error_left + 0.001f < error_right && error_left + 0.001f < error_middle)
		{
			t_right = t_middle;
			t_middle = t_left_middle;
			error_middle = error_left;
		}
		else if (error_middle + 0.001f < error_right && error_middle + 0.001f < error_left)
		{
			t_right = t_right_middle;
			t_left = t_left_middle;
		}
		else if (error_right + 0.001f < error_middle && error_right + 0.001f < error_left)
		{
			t_left = t_middle;
			t_middle = t_right_middle;
			error_middle = error_right;
		}
		else
		{
			break;//2 are equals
		}
		t_left_middle = (t_left + t_middle)*0.5f;
		t_right_middle = (t_right + t_middle)*0.5f;
		++iter;
	} while (iter < max_iter);// t_left_middle>0.1 && t_right_middle < 0.9);
	return fast_length(bezier_pt_deg2(ctrl_point,t_middle)- point);
}
float get_error_deg3(float2 point, __read_only __global float2* ctrl_point)
{
	float error;
	float t_left = 0.0f, t_right = 1.0f;
	float t_middle = 0.5f;
	float t_left_middle = 0.25f;
	float t_right_middle = 0.75f;

	float error_middle = fast_length(bezier_pt_deg3(ctrl_point,t_middle)- point);
	float error_left, error_right;
	int max_iter = 100;
	int iter = 0;
	do
	{
		error_left = fast_length(bezier_pt_deg3(ctrl_point,t_left_middle)- point);
		error_right = fast_length(bezier_pt_deg3(ctrl_point,t_right_middle)- point);//fn(t_right_middle);
		if (error_left + 0.001f < error_right && error_left + 0.001f < error_middle)
		{
			t_right = t_middle;
			t_middle = t_left_middle;
			error_middle = error_left;
		}
		else if (error_middle + 0.001f < error_right && error_middle + 0.001f < error_left)
		{
			t_right = t_right_middle;
			t_left = t_left_middle;
		}
		else if (error_right + 0.001f < error_middle && error_right + 0.001f < error_left)
		{
			t_left = t_middle;
			t_middle = t_right_middle;
			error_middle = error_right;
		}
		else
		{
			break;//2 are equals
		}
		t_left_middle = (t_left + t_middle)*0.5f;
		t_right_middle = (t_right + t_middle)*0.5f;
		++iter;
	} while (iter < max_iter);// t_left_middle>0.1 && t_right_middle < 0.9);
	return fast_length(bezier_pt_deg3(ctrl_point,t_middle)- point);
}
__kernel void bezier_error (__read_only __global float2* ref_point,
 __read_only __global float2* test_data, __read_only __global int4* data_select,
							  __read_write __global int* min_dst)
{
	const int4 id = data_select[get_global_id(0)];
	const float2 ptref = ref_point[id.x];
	__read_only __global float2* ctrl_point_test = test_data + id.y;
	switch(id.z)
	{
	case 2:
		atomic_min(min_dst + id.w,convert_int(10000.0f*get_error_deg1(ptref,ctrl_point_test)));
	break;
	case 3:
		atomic_min(min_dst + id.w,convert_int(10000.0f*get_error_deg2(ptref,ctrl_point_test)));
	break;
	case 4:
		atomic_min(min_dst + id.w,convert_int(10000.0f*get_error_deg3(ptref,ctrl_point_test)));
	break;
	}
}






