

float2 bezier_pt_deg1(__read_only __global float2* ctrl_point, const float t)
{
	return ctrl_point[0] * (1.0f-t) + ctrl_point[1] * t;
}
float2 bezier_pt_deg2(__read_only __global float2* V, const float t)
{
		const float t2 = t*t;
		const float t12 = 2.0f*t;
	return V[0] * (t2 - t12 + 1.0f) + V[1] * (t12 - 2.0f * t2) + V[2] * t2;
}
float2 bezier_pt_deg3(__read_only __global float2* V, const float t)
{
		const float t2 = t*t;
		const float t3 = t2*t;
		const float _3t = t * 3.0f;
		const float _3t2 = t2 * 3.0f;
		const float _3t3 = t3 * 3.0f;
	return V[0] * (-t3 + _3t2 - _3t + 1.0f) + V[1] * (_3t3 - 2.0f * _3t2 + _3t) + V[2] * (-_3t3 + _3t2) + V[3] * t3;
}


__kernel void bezier_length(__read_only __global float2* ref_data,
 __read_only __global int* ref_data_idx_and_size,__write_only __global float* curve_length, const int nb_curve, const float inv_dt)
{
	const int idx = get_global_id(0);
	
	const int id_data = ref_data_idx_and_size[get_global_id(0)<<1];
	__read_only __global float2* ctrl_point = ref_data + id_data;
	const int nb_elem = ref_data_idx_and_size[(get_global_id(0)<<1)+1];
	float2 p1, p2;
	float length = 0.0f;
	const float inv_inv_dt = 1.0f/inv_dt;
	
	switch(nb_elem)
	{
	case 2:
		p1 = bezier_pt_deg1(ctrl_point, 0.0f);
		for (int i = 1, i_end = convert_int(inv_dt); i != i_end; ++i)
		{
			p2 = bezier_pt_deg1(ctrl_point, convert_float(i)*inv_inv_dt);
			length += distance(p1, p2);
			p1=p2;
		}
	break;
	case 3:
		p1 = bezier_pt_deg2(ctrl_point, 0.0f);
		for (int i = 1, i_end = convert_int(inv_dt); i != i_end; ++i)
		{
			p2 = bezier_pt_deg2(ctrl_point, convert_float(i)*inv_inv_dt);
			length += distance(p1, p2);
			p1=p2;
		}
	break;
	case 4:
		p1 = bezier_pt_deg3(ctrl_point, 0.0f);
		for (int i = 1, i_end = convert_int(inv_dt); i != i_end; ++i)
		{
			p2 = bezier_pt_deg3(ctrl_point, convert_float(i)*inv_inv_dt);
			length += distance(p1, p2);
			p1=p2;
		}
	break;
	}
	curve_length[get_global_id(0)]=length;//convert_int(length);
}
